import textwrap
from io import StringIO
from unittest import TestCase

from htimeseries import HTimeseries

from haggregate import AggregateError, aggregate

tenmin_test_timeseries = textwrap.dedent(
    """\
            2008-02-07 09:40,10.32,
            2008-02-07 09:50,10.42,
            2008-02-07 10:00,10.51,
            2008-02-07 10:10,10.54,
            2008-02-07 10:20,10.71,
            2008-02-07 10:30,10.96,
            2008-02-07 10:40,10.93,
            2008-02-07 10:50,11.10,
            2008-02-07 11:00,11.23,
            2008-02-07 11:10,11.44,
            2008-02-07 11:20,11.41,
            2008-02-07 11:30,11.42,MISS
            2008-02-07 11:40,11.54,
            2008-02-07 11:50,11.68,
            2008-02-07 12:00,11.80,
            2008-02-07 12:10,11.91,
            2008-02-07 12:20,12.16,
            2008-02-07 12:30,12.16,
            2008-02-07 12:40,12.24,
            2008-02-07 12:50,12.13,
            2008-02-07 13:00,12.17,
            2008-02-07 13:10,12.31,
            """
)


tenmin_allmiss_test_timeseries = textwrap.dedent(
    """\
            2005-05-01 00:10,,
            2005-05-01 00:20,,
            2005-05-01 00:30,,
            2005-05-01 00:40,,
            2005-05-01 00:50,,
            2005-05-01 01:00,,
            2005-05-01 01:10,,
            2005-05-01 01:20,1,
            2005-05-01 01:30,,
            2005-05-01 01:40,1,
            2005-05-01 01:50,,
            2005-05-01 02:00,1,
            """
)


aggregated_hourly_allmiss = textwrap.dedent(
    """\
            2005-05-01 02:00,3,MISS\r
            """
)


class HourlySumTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_test_timeseries))
        self.result = aggregate(self.ts, "1H", "sum", min_count=3, missing_flag="MISS")

    def test_length(self):
        self.assertEqual(len(self.result.data), 4)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:00"].value, 31.25)
        self.assertEqual(self.result.data["flags"].loc["2008-02-07 10:00"], "MISS")

    def test_value_2(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 11:00"].value, 65.47)

    def test_value_3(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 12:00"].value, 69.29)

    def test_value_4(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 13:00"].value, 72.77)


class TimestampIrregularityTestCase(TestCase):
    def test_raises_exception(self):
        # In the time series below, it's 10:41 instead of 10:40
        ts = HTimeseries(
            StringIO(
                textwrap.dedent(
                    """\
                   2008-02-07 10:10,10.54,
                   2008-02-07 10:20,10.71,
                   2008-02-07 10:30,10.96,
                   2008-02-07 10:41,10.93,
                   2008-02-07 10:50,11.10,
                   2008-02-07 11:00,11.23,
                   """
                )
            )
        )
        with self.assertRaises(AggregateError):
            aggregate(ts, "1H", "sum")


class WrongTimeStepTestCase(TestCase):
    def test_raises_exception(self):
        msg = "The target step can currently only be 1H or 1D"
        with self.assertRaisesRegex(AggregateError, msg):
            aggregate(HTimeseries(), "5D", "sum")


class HourlySumWithLargerMinCountTestCase(TestCase):
    """Same as HourlySumTestCase but with slightly larger min_count.
    """

    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_test_timeseries))
        self.result = aggregate(self.ts, "1H", "sum", min_count=4, missing_flag="MISS")

    def test_length(self):
        self.assertEqual(len(self.result.data), 3)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 11:00"].value, 65.47)

    def test_value_2(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 12:00"].value, 69.29)

    def test_value_3(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 13:00"].value, 72.77)


class HourlyMeanTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_test_timeseries))
        self.result = aggregate(self.ts, "1H", "mean", min_count=3, missing_flag="MISS")

    def test_length(self):
        self.assertEqual(len(self.result.data), 4)

    def test_value_1(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 10:00"].value, 10.4166667
        )

    def test_value_2(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 11:00"].value, 10.9116667
        )

    def test_value_3(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 12:00"].value, 11.5483333
        )

    def test_value_4(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 13:00"].value, 12.1283333
        )


class HourlyMeanWithOffsetTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_test_timeseries))
        self.result = aggregate(
            self.ts,
            "1H",
            "mean",
            min_count=3,
            missing_flag="MISS",
            target_timestamp_offset="1min",
        )

    def test_length(self):
        self.assertEqual(len(self.result.data), 4)

    def test_value_1(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 09:59"].value, 10.4166667
        )

    def test_value_2(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 10:59"].value, 10.9116667
        )

    def test_value_3(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 11:59"].value, 11.5483333
        )

    def test_value_4(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 12:59"].value, 12.1283333
        )


class HourlyMeanWithNegativeOffsetTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_test_timeseries))
        self.result = aggregate(
            self.ts,
            "1H",
            "mean",
            min_count=3,
            missing_flag="MISS",
            target_timestamp_offset="-1min",
        )

    def test_length(self):
        self.assertEqual(len(self.result.data), 4)

    def test_value_1(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 10:01"].value, 10.4166667
        )

    def test_value_2(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 11:01"].value, 10.9116667
        )

    def test_value_3(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 12:01"].value, 11.5483333
        )

    def test_value_4(self):
        self.assertAlmostEqual(
            self.result.data.loc["2008-02-07 13:01"].value, 12.1283333
        )


class HourlyMaxTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_test_timeseries))
        self.result = aggregate(self.ts, "1H", "max", min_count=3, missing_flag="MISS")

    def test_length(self):
        self.assertEqual(len(self.result.data), 4)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:00"].value, 10.51)

    def test_value_2(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 11:00"].value, 11.23)

    def test_value_3(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 12:00"].value, 11.8)

    def test_value_4(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 13:00"].value, 12.24)


class HourlyMinTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_test_timeseries))
        self.result = aggregate(self.ts, "1H", "min", min_count=3, missing_flag="MISS")

    def test_length(self):
        self.assertEqual(len(self.result.data), 4)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:00"].value, 10.32)

    def test_value_2(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 11:00"].value, 10.54)

    def test_value_3(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 12:00"].value, 11.41)

    def test_value_4(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 13:00"].value, 11.91)


class AggregateEmptyTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries()
        self.result = aggregate(self.ts, "1H", "sum", min_count=3, missing_flag="MISS")

    def test_length(self):
        self.assertEqual(len(self.result.data), 0)


class AllMissAggregateTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_allmiss_test_timeseries))
        self.result = aggregate(self.ts, "1H", "sum", min_count=1, missing_flag="MISS")

    def test_length(self):
        self.assertEqual(len(self.result.data), 1)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2005-05-01 02:00"].value, 3)


class SetsMetadataTestCase(TestCase):
    def setUp(self):
        self.ts = HTimeseries(StringIO(tenmin_test_timeseries))
        self.ts.title = "hello"
        self.ts.precision = 1
        self.ts.comment = "world"
        self.ts.timezone = "EET (+0200)"
        self.result = aggregate(self.ts, "1H", "sum", min_count=3, missing_flag="MISS")

    def test_sets_title(self):
        self.assertEqual(self.result.title, "Aggregated hello")

    def test_sets_precision(self):
        self.assertEqual(self.result.precision, 1)

    def test_sets_comment(self):
        self.assertEqual(
            self.result.comment,
            "Created by aggregating the time series that had this comment:\n\nworld",
        )

    def test_sets_time_step(self):
        self.assertEqual(self.result.time_step, "1H")

    def test_sets_timezone(self):
        self.assertEqual(self.result.timezone, "EET (+0200)")



import re

import numpy as np
import pandas as pd
from htimeseries import HTimeseries

methods = {
    "sum": pd.Series.sum,
    "mean": pd.Series.mean,
    "max": pd.Series.max,
    "min": pd.Series.min,
}


class AggregateError(Exception):
    pass


def aggregate(
    hts,
    target_step,
    method,
    min_count=1,
    missing_flag="MISS",
    target_timestamp_offset=None,
):
    result = HTimeseries()

    # Set metadata of result
    result = HTimeseries()
    attrs = ("unit", "timezone", "interval_type", "variable", "precision", "location")
    for attr in attrs:
        setattr(result, attr, getattr(hts, attr, None))
    if target_step not in ("1H", "1D"):
        raise AggregateError("The target step can currently only be 1H or 1D")
    result.time_step = target_step
    if hasattr(hts, "title"):
        result.title = "Aggregated " + hts.title
    if hasattr(hts, "comment"):
        result.comment = (
            "Created by aggregating the time series that had this comment:\n\n"
            + hts.comment
        )

    # Reindex the source so that it has no missing records but has NaNs instead,
    # starting from one before and ending in one after
    current_range = hts.data.index
    try:
        freq = pd.tseries.frequencies.to_offset(pd.infer_freq(current_range))
        if freq is None:
            raise AggregateError(
                "Can't infer time series step; maybe it's not regularized"
            )
    except ValueError:
        # Can't infer frequency - insufficient number of records
        return result
    first_timestamp = (current_range[0] - pd.Timedelta("1S")).floor(target_step)
    end_timestamp = current_range[-1].ceil(target_step)
    new_range = pd.date_range(first_timestamp, end_timestamp, freq=freq)
    source_data = hts.data.reindex(new_range)
    # Do the resampling
    resampler = source_data["value"].resample(
        target_step, closed="right", label="right"
    )
    result_values = resampler.agg(methods[method])

    # Convert to NaN when there aren't enough source records
    values_count = resampler.count()
    result_values[values_count < min_count] = np.nan
    result.data["value"] = result_values

    # Set the missing flag wherever the source has a missing value and the target has
    # a value
    max_count = int(pd.Timedelta(target_step) / freq)
    result.data["flags"] = (values_count < max_count).apply(
        lambda x: missing_flag if x else ""
    )

    # Remove leading and trailing NaN values from the result
    while pd.isnull(result.data["value"]).iloc[0]:
        result.data = result.data.drop(result.data.index[0])
    while pd.isnull(result.data["value"]).iloc[-1]:
        result.data = result.data.drop(result.data.index[-1])

    # Add timestamp_offset
    if target_timestamp_offset:
        periods = target_timestamp_offset.startswith("-") and 1 or -1
        freq = target_timestamp_offset.lstrip("-")
        result.data = result.data.shift(periods, freq=freq)

    return result


def _get_offset_in_minutes(timestamp_offset):
    m = re.match(r"(-?)(\d*)(T|min)$", timestamp_offset)
    if not m:
        raise AggregateError(
            "The target timestamp offset can currently only be a number of minutes "
            "such as 1min"
        )
    sign = m.group(1) == "-" and -1 or 1
    return sign * int(m.group(2))



import textwrap
from io import StringIO
from unittest import TestCase

import numpy as np
from htimeseries import HTimeseries

from haggregate import RegularizeError, regularize


class BadTimeStepTestCase(TestCase):
    def test_unspecified_time_step(self):
        ts = HTimeseries()
        msg = "The source time series does not specify a time step"
        with self.assertRaisesRegex(RegularizeError, msg):
            regularize(ts)

    def test_malformed_time_step(self):
        ts = HTimeseries()
        ts.time_step = "hello"
        msg = (
            "The time step is malformed or is specified in months. Only time steps "
            "specified in minutes, hours or days are supported."
        )
        with self.assertRaisesRegex(RegularizeError, msg):
            regularize(ts)

    def test_malformed_time_step2(self):
        ts = HTimeseries()
        ts.time_step = "5M"
        msg = (
            "The time step is malformed or is specified in months. Only time steps "
            "specified in minutes, hours or days are supported."
        )
        with self.assertRaisesRegex(RegularizeError, msg):
            regularize(ts)


class RegularizeTestCase(TestCase):
    def setUp(self):
        input = textwrap.dedent(
            """\
            2008-02-07 10:30,10.71,FLAG1
            2008-02-07 10:41,10.93,FLAG2
            2008-02-07 10:50,11.10,
            """
        )
        ts = HTimeseries(StringIO(input))
        ts.time_step = "10min"
        self.result = regularize(ts)

    def test_length(self):
        self.assertEqual(len(self.result.data), 3)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:30"].value, 10.71)

    def test_value_2(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:40"].value, 10.93)

    def test_value_3(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:50"].value, 11.10)

    def test_flags_1(self):
        self.assertEqual(self.result.data.loc["2008-02-07 10:30"]["flags"], "FLAG1")

    def test_flags_2(self):
        self.assertEqual(
            self.result.data.loc["2008-02-07 10:40"]["flags"], "FLAG2 DATEINSERT"
        )

    def test_flags_3(self):
        self.assertEqual(self.result.data.loc["2008-02-07 10:50"]["flags"], "")


class RegularizeFirstRecordTestCase(TestCase):
    def setUp(self):
        input = textwrap.dedent(
            """\
            2008-02-07 10:31,10.71,
            2008-02-07 10:40,10.93,
            2008-02-07 10:50,11.10,
            """
        )
        ts = HTimeseries(StringIO(input))
        ts.time_step = "10min"
        self.result = regularize(ts)

    def test_length(self):
        self.assertEqual(len(self.result.data), 3)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:30"].value, 10.71)

    def test_value_2(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:40"].value, 10.93)

    def test_value_3(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:50"].value, 11.10)


class RegularizeLastRecordTestCase(TestCase):
    def setUp(self):
        input = textwrap.dedent(
            """\
            2008-02-07 10:30,10.71,
            2008-02-07 10:40,10.93,
            2008-02-07 10:49,11.10,
            """
        )
        ts = HTimeseries(StringIO(input))
        ts.time_step = "10min"
        self.result = regularize(ts)

    def test_length(self):
        self.assertEqual(len(self.result.data), 3)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:30"].value, 10.71)

    def test_value_2(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:40"].value, 10.93)

    def test_value_3(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:50"].value, 11.10)


class RegularizeNullRecordTestCase(TestCase):
    def setUp(self):
        input = textwrap.dedent(
            """\
            2008-02-07 10:30,10.71,
            2008-02-07 10:51,11.10,
            2008-02-07 11:00,10.93,
            """
        )
        ts = HTimeseries(StringIO(input))
        ts.time_step = "10min"
        self.result = regularize(ts)

    def test_length(self):
        self.assertEqual(len(self.result.data), 4)

    def test_value_1(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:30"].value, 10.71)

    def test_value_2(self):
        self.assertTrue(np.isnan(self.result.data.loc["2008-02-07 10:40"].value))

    def test_value_3(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 10:50"].value, 11.10)

    def test_value_4(self):
        self.assertAlmostEqual(self.result.data.loc["2008-02-07 11:00"].value, 10.93)


class RegularizeEmptyTestCase(TestCase):
    def setUp(self):
        ts = HTimeseries()
        ts.time_step = "10min"
        self.result = regularize(ts)

    def test_length(self):
        self.assertEqual(len(self.result.data), 0)


class SetsMetadataTestCase(TestCase):
    def setUp(self):
        input = textwrap.dedent(
            """\
            2008-02-07 10:30,10.71,FLAG1
            2008-02-07 10:41,10.93,FLAG2
            2008-02-07 10:50,11.10,
            """
        )
        self.ts = HTimeseries(StringIO(input))
        self.ts.time_step = "10min"
        self.ts.title = "hello"
        self.ts.precision = 1
        self.ts.comment = "world"
        self.ts.timezone = "EET (+0200)"
        self.result = regularize(self.ts)

    def test_sets_title(self):
        self.assertEqual(self.result.title, "Regularized hello")

    def test_sets_precision(self):
        self.assertEqual(self.result.precision, 1)

    def test_sets_comment(self):
        self.assertEqual(
            self.result.comment,
            "Created by regularizing step of timeseries "
            "that had this comment:\n\nworld",
        )

    def test_sets_time_step(self):
        self.assertEqual(self.result.time_step, "10min")

    def test_sets_timezone(self):
        self.assertEqual(self.result.timezone, "EET (+0200)")



import textwrap
from unittest import TestCase
from unittest.mock import patch

from click.testing import CliRunner

from haggregate import cli


class CliUsageErrorTestCase(TestCase):
    def setUp(self):
        runner = CliRunner()
        self.result = runner.invoke(cli.main, [])

    def test_exit_code(self):
        self.assertTrue(self.result.exit_code > 0)

    def test_error_message(self):
        self.assertIn("Usage: main [OPTIONS] CONFIGFILE", self.result.output)


class CliConfigFileNotFoundTestCase(TestCase):
    def setUp(self):
        runner = CliRunner()
        self.result = runner.invoke(cli.main, ["/nonexistent/nonexistent"])

    def test_exit_code(self):
        self.assertTrue(self.result.exit_code > 0)

    def test_error_message(self):
        self.assertIn("No such file or directory", self.result.output)


class CliNoTimeSeriesErrorTestCase(TestCase):
    def setUp(self):
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open("config.ini", "w") as f:
                f.write(
                    textwrap.dedent(
                        """\
                        [General]
                        target_step = D
                        min_count = 10
                        missing_flag = MISSING
                        """
                    )
                )
            self.result = runner.invoke(cli.main, ["config.ini"])

    def test_exit_code(self):
        self.assertTrue(self.result.exit_code > 0)

    def test_error_messages(self):
        self.assertIn("No time series have been specified", self.result.output)


class CliMixin:
    @patch("haggregate.cli.HTimeseries", **{"return_value": "my timeseries"})
    @patch("haggregate.cli.regularize", return_value="regularized timeseries")
    @patch("haggregate.cli.aggregate")
    def setUp(self, mock_aggregate, mock_regularize, mock_htimeseries):
        self.mock_aggregate = mock_aggregate
        self.mock_regularize = mock_regularize
        self.mock_htimeseries = mock_htimeseries
        runner = CliRunner()
        with runner.isolated_filesystem():
            with open("config.ini", "w") as f:
                f.write(self.configuration)
            open("mytimeseries.hts", "a").close()
            self.result = runner.invoke(cli.main, ["config.ini"])


class CliTestCase(CliMixin, TestCase):
    configuration = textwrap.dedent(
        """\
        [General]
        target_step = D
        min_count = 10
        missing_flag = MISSING

        [MyTimeseries]
        source_file = mytimeseries.hts
        target_file = aggregatedtimeseries.hts
        method = sum
        """
    )

    def test_exit_code(self):
        self.assertEqual(self.result.exit_code, 0)

    def test_read_source_file(self):
        self.assertEqual(self.mock_htimeseries.call_count, 1)

    def test_regularize_called_correctly(self):
        self.mock_regularize.assert_called_once_with(
            "my timeseries", new_date_flag="DATEINSERT"
        )

    def test_aggregate_called_correctly(self):
        self.mock_aggregate.assert_called_once_with(
            "regularized timeseries",
            "D",
            "sum",
            min_count=10,
            missing_flag="MISSING",
            target_timestamp_offset=None,
        )

    def test_wrote_target_file(self):
        self.assertEqual(self.mock_aggregate.return_value.write.call_count, 1)


class CliWithTargetTimestampOffsetTestCase(CliMixin, TestCase):
    configuration = textwrap.dedent(
        """\
        [General]
        target_step = D
        target_timestamp_offset = 1min
        min_count = 10
        missing_flag = MISSING

        [MyTimeseries]
        source_file = mytimeseries.hts
        target_file = aggregatedtimeseries.hts
        method = sum
        """
    )

    def test_exit_code(self):
        self.assertEqual(self.result.exit_code, 0)

    def test_read_source_file(self):
        self.assertEqual(self.mock_htimeseries.call_count, 1)

    def test_regularize_called_correctly(self):
        self.mock_regularize.assert_called_once_with(
            "my timeseries", new_date_flag="DATEINSERT"
        )

    def test_aggregate_called_correctly(self):
        self.mock_aggregate.assert_called_once_with(
            "regularized timeseries",
            "D",
            "sum",
            min_count=10,
            missing_flag="MISSING",
            target_timestamp_offset="1min",
        )

    def test_wrote_target_file(self):
        self.assertEqual(self.mock_aggregate.return_value.write.call_count, 1)



#!/usr/bin/env python

import os
import re

from setuptools import Extension, find_packages, setup

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = ["Click>=7.0,<8", "htimeseries>=2.0.3,<3"]

setup_requirements = ["cython>=0.29,<0.30"]

test_requirements = []


def use_cython():
    base_dir = os.path.dirname(os.path.realpath(__file__))

    regularize_pyx = os.path.join(base_dir, "haggregate", "regularize.pyx")
    regularize_pyx_exists = os.path.exists(regularize_pyx)
    regularize_c = os.path.join(base_dir, "haggregate", "regularize.c")
    regularize_c_exists = os.path.exists(regularize_c)

    if (not regularize_pyx_exists) and (not regularize_c_exists):
        raise Exception("Neither {} nor {} exists".format(regularize_pyx, regularize_c))

    return (not regularize_c_exists) or (
        regularize_pyx_exists
        and os.path.getmtime(regularize_pyx) > os.path.getmtime(regularize_c)
    )


if use_cython():
    import numpy
    from Cython.Build import cythonize

    ext_modules = cythonize(
        "haggregate/regularize.pyx", include_path=[numpy.get_include()]
    )
else:
    import numpy

    ext_modules = [
        Extension(
            "haggregate.regularize",
            ["haggregate/regularize.c"],
            include_dirs=[numpy.get_include()],
        )
    ]


def get_version():
    scriptdir = os.path.dirname(os.path.abspath(__file__))
    init_py_path = os.path.join(scriptdir, "haggregate", "__init__.py")
    with open(init_py_path) as f:
        return re.search(r'^__version__ = "(.*?)"$', f.read(), re.MULTILINE).group(1)


setup(
    ext_modules=ext_modules,
    author="Antonis Christofides",
    author_email="antonis@antonischristofides.com",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    description="Aggregates htimeseries to larger steps",
    entry_points={"console_scripts": ["haggregate=haggregate.cli:main"]},
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + "\n\n" + history,
    include_package_data=True,
    keywords="haggregate",
    name="haggregate",
    packages=find_packages(include=["haggregate"]),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/openmeteo/haggregate",
    version=get_version(),
    zip_safe=False,
)



import configparser
import datetime as dt
import logging
import os
import sys
import traceback

import click
from htimeseries import HTimeseries

from haggregate import aggregate
from haggregate.regularize import regularize


@click.command()
@click.argument("configfile")
def main(configfile):
    """Create lower-step timeseries from higher-step ones"""

    # Start by setting logger to stdout; later we will switch it according to config
    logger = logging.getLogger("haggregate")
    stdout_handler = logging.StreamHandler()
    logger.addHandler(stdout_handler)

    try:
        config = configparser.ConfigParser()
        with open(configfile) as f:
            config.read_file(f)

        # Read the [General] section
        logfile = config.get("General", "logfile", fallback="")
        loglevel = config.get("General", "loglevel", fallback="warning")
        base_dir = config.get("General", "base_dir", fallback=".")
        target_step = config.get("General", "target_step")
        min_count = config.getint("General", "min_count")
        missing_flag = config.get("General", "missing_flag")
        target_timestamp_offset = config.get(
            "General", "target_timestamp_offset", fallback=None
        )

        # Remove [General] and make sure there are more sections
        config.pop("General")
        if not len(config.sections()):
            raise configparser.NoSectionError("No time series have been specified")

        # Setup logger
        logger.setLevel(loglevel.upper())
        if logfile:
            logger.removeHandler(stdout_handler)
            logger.addHandler(logging.FileHandler(logfile))

        # Log start of execution
        logger.info("Starting haggregate, " + dt.datetime.today().isoformat())

        # Read each section and do the work for it
        for section_name in config.sections():
            section = config[section_name]
            source_filename = os.path.join(base_dir, section.get("source_file"))
            target_filename = os.path.join(base_dir, section.get("target_file"))
            method = section.get("method")
            with open(source_filename, newline="\n") as f:
                ts = HTimeseries(f, format=HTimeseries.FILE)
            regts = regularize(ts, new_date_flag="DATEINSERT")
            aggts = aggregate(
                regts,
                target_step,
                method,
                min_count=min_count,
                missing_flag=missing_flag,
                target_timestamp_offset=target_timestamp_offset,
            )
            with open(target_filename, "w") as f:
                aggts.write(f, format=HTimeseries.FILE)

        # Log end of execution
        logger.info("Finished haggregate, " + dt.datetime.today().isoformat())

    except Exception as e:
        logger.error(str(e))
        logger.debug(traceback.format_exc())
        raise click.ClickException(str(e))


if __name__ == "__main__":
    sys.exit(main())
